#!/usr/bin/python
# -*- coding: utf-8 -*-
# Version 5.3
# 2011-03-17 - Plan CEIBAL
# Autor: 
# Modificado por 
# Modificado por 
#
# 
# 
#

import os
import time
import commands
import random
import tarfile
import re
import shutil
import logging
import logging.handlers

from ConfigParser import ConfigParser, NoSectionError

from ceibal import env
from ceibal import util
from ceibal import security
from ceibal.laptops import XO
from ceibal.laptops import MG2

###########################################
# constants
###########################################
TMP_DIR = "/tmp/actualizaciones"
WORK_DIR = env.get_updates_root()

class ActMG2:
    
    def __init__(self):
        self._laptop = MG2()
        self._laptop.chk_default_update_sections()
        self._update_list = 'actualizaciones.list'
        self._updated_today = os.path.join(WORK_DIR, "acthoy")
        self._port = 5000
        try:
            self._server = util.get_gateway()
        except:
            self._server = None
        self.__set_logger()
        self.__send_update_info()

    def __set_logger(self):
        self._logger = logging.getLogger('actualizador')
        self._logger.setLevel(logging.DEBUG)
        # Add the log message handler to the logger
        handler = logging.FileHandler(os.path.join(WORK_DIR, "actualizador.log"), 
                                        'w') #, maxBytes= 300, backupCount= 1)
        self._logger.addHandler(handler)

    def __send_update_info(self):
        first_act = os.path.join(WORK_DIR,"first_act")
        if not os.path.exists(first_act):
            versions = self.get_current_version()
            for version in versions:
                self.__send_act(version, versions[version])
            open(first_act, 'w').close()

    def __send_data(self, msj):
        """
        Envia la informacion de la MG2 hacia el servidor
        """
        print "Entramos al send_data"
        self._laptop.send_data(self._server, self._port, msj)

    def __send_act(self, categoria, version):
        """
        Envia la informacion de la actualizacion indicada
        de la MG2 hacia el servidor
        """
        self._laptop.send_act(self._server, self._port, categoria, version)

    def __send_error(self, categoria, version, msg):
        """
        Envia la informacion del error ocurrido durante la actualizacion.
        """
        self._laptop.send_error(self._server, self._port, categoria, version, msg)

    def __set_update_today(self):
        """
        Marca que hoy se realizo una actualizacion
        """
        util.data_2_file(time.strftime('%Y%m%d', time.localtime()), self._updated_today)

    def get_current_version(self):
        """
        Retorna un diccionario con las versiones actuales de cada actualizacion para la MG2
        """
        return self._laptop.get_current_update_version()

    def __install_update(self, categoria, version):

        """
        Instala la actualizacion en la categoria indicada
        """
        util.clean_dir(TMP_DIR)
        error = False
        # el nombre del archivo de actualizacion debe ser ceibal-<aaammdd>-<categoria>.tar.gz
        nombre_archivo = 'ceibal-%s-%s.tar.gz' %(categoria, version)
        filename = os.path.join('/','actualizaciones',categoria,nombre_archivo)
        self._logger.debug(time.ctime() + '-Intentando descargar %s' %nombre_archivo)
        data = util.download_file(self._server,filename , os.path.join(TMP_DIR,
                                                                    nombre_archivo))
        data = util.download_file(self._server, filename + ".sig",
                                    os.path.join(TMP_DIR, nombre_archivo + '.sig'))
        if not util.chk_signature(os.path.join(TMP_DIR, '%s.sig' %(nombre_archivo))):
            msg = 'ERROR: Firma de %s INVALIDA. Saliendo...' %nombre_archivo
            error = True
            return error, msg
        arch_tar = tarfile.open(os.path.join(TMP_DIR, nombre_archivo))
        arch_tar.extractall(TMP_DIR)
        if self.__chk_install():
            if self.__chk_deps(os.path.join(TMP_DIR, "deps.list")):
                os.chdir(TMP_DIR)
                info = commands.getstatusoutput('sh instalar')
                os.chdir(WORK_DIR)
                if info[0] > 0:
                    msg = '\t' + info[1]
                else:
                    msg = "actualizacion instalada"
                    self._laptop.set_current_update_version(categoria,version)
                    self.__send_act(categoria, version)
            else:
                msg = 'La actualizacion (%s) %s no se instalo: faltan dependencias' \
                                                                %(categoria,version)
        else:
            msg = 'La actualizacion (%s) %s no corresponde a esta MG2' %(categoria,version)
        return error, msg

    def already_checked_for_act(self):
        """
        Retorna True si ya se realizo un chequeo hoy
        """
        try:
            f = open(self._updated_today, 'r')
        except:
            return False
        dia = f.read()
        f.close()
        hoy = time.strftime("%Y%m%d", time.localtime())
        return dia == hoy

    def __chk_deps(self, file): #Agrego metodo chk_deps
        """
        Revisa dependencias de la actualizacion
        @file: archivo con las dependencias de la actualizacion, el formato es
                categoria:version
                categoria:version
                .
                .
                .
        """
        result = True
        try:
            lines = open(file).readlines()
            while result and (len(lines) > 0):
                line = lines.pop()
                tuple = line.split(":")
                category = tuple[0].strip()
                version = int(tuple[1].strip())
                result = version <= int(self._laptop.get_update_version(category))
        except IOError:
            result = True
        return result


    def get_list_available_updates(self,installed_updates, available_updates):
        """
        @installed_updates: Diccionario con la ultima version en cada categoria
        @available_updates: Archivo con las actualizaciones disponibles en cada categoria
        """
        available = ConfigParser()
        available.read(available_updates)
        versiones = {}
        for seccion in installed_updates.keys():
            try:
                list = map(lambda x: x[1], available.items(seccion))
                mi_version = installed_updates[seccion]
                # me quedo con las versiones mayores a la mia
                list = filter(lambda x: int(x) > int(mi_version), list)
                versiones[seccion] = sorted(list)
            except NoSectionError:
                pass
        return versiones

    def __chk_mac_wireless_install(self, list_mac_wireless):
        """
        Revisa si la actualizacion corresponde al mac_wireless intenta instalarla.
        """
        result = False
        try:
            arch = open(list_mac_wireless)
            list_mac_wireless = map(lambda x: x.strip("\n"),arch.readlines())
            result = self._laptop._mac_wireless in list_mac_wireless
        except IOError:
            result = True
        except:
            result = False

        return result


    def __chk_conf_install(self, conf_file):
        """
        Revisa si la actualizacion corresponde a la configuracion
        de la MG2 que intenta instalarla.
        """
        result = True
        try:
            arch = open(conf_file)
            list_conf = map(lambda x: x.strip("\n"),arch.readlines())
            active_cats = self._laptop.get_current_active_updates()
            for conf in list_conf:
                result = result and (conf in active_cats)
            result = self._laptop._mac_wireless in list_mac_wireless
        except IOError:
            result = True
        except:
            result = False
        return result

    def __chk_install(self): #Agrego el metodo chk_install
        """
        Revisa si la actualizacion corresponde a la MG2 que intenta instalarla.
        """
        list_mac_wireless = os.path.join(TMP_DIR, "mac_wireless.list")
        extra_conf = os.path.join(TMP_DIR, "extra.conf")
        return self.__chk_mac_wireless_install(list_mac_wireless) and self.__chk_conf_install(extra_conf)


    def run(self):
        """
        Busca las actualizaciones disponibles para las categorias habilitadas,
        baja e instala las actualizaciones disponibles
        """
        os.chdir(TMP_DIR)
        if self.already_checked_for_act():
            self._logger.debug(time.ctime() + '-AVISO: Ya hubo un chequeo hoy. Saliendo...')
            exit()


        self._logger.debug(time.ctime() + '-Esperando un tiempo aleatorio...')
        time.sleep(random.randint(0, 120))

        if not self._server:
            self._logger.debug(time.ctime() + '-No se pudo obtener una direccion para el servidor. \
Saliendo...')
            exit()

        self._logger.debug(time.ctime() + '-Chequeando conexion al servidor (%s)...' %self._server)

        if not util.chk_connection(self._server):
            self._logger.debug(time.ctime() + '-ERROR: No existe ruta al servidor. Saliendo')
            exit()
        self._logger.debug(time.ctime() + '-Conexion establecida con %s' %self._server)
        self.__send_data(0)


        self._logger.debug(time.ctime() + '-Intentando descargar lista de actualizaciones...')
        update_list = util.download_file(self._server,'/actualizaciones/'+\
                    self._update_list, os.path.join(TMP_DIR, self._update_list))

        if not update_list:
            self._logger.debug(time.ctime() + '-ERROR: No se pudo descargar la lista de \
actualizaciones. Saliendo...')
            exit()
        self._logger.debug(time.ctime() + '-Se obtuvo la lista de actualizaciones.')
        self._logger.debug(time.ctime() + '-Buscando actualizaciones faltantes...')
        faltantes = self.get_list_available_updates(self.get_current_version(),
                                        os.path.join(TMP_DIR, self._update_list))
        if not len(faltantes):
            self._logger.debug(time.ctime() + '-No se necesita actualizar. Saliendo...')
            self.__set_update_today()
            exit()

        for categoria in faltantes.keys():
            if not len(faltantes[categoria]):
                pass
            else:
                self._logger.debug(time.ctime() + '-Se van a instalar %i actualizaciones en %s.' %
                                        (faltantes[categoria].__len__(),categoria))
                for version in faltantes[categoria]:
                    error, msg = self.__install_update(categoria, version)
                    self._logger.debug(msg)
                    if error:
                        self.__send_error(categoria, version,
                                        "No se pudo instalar la actualizacion")
        self._logger.debug(time.ctime() + '-Actualizaciones finalizadas. Saliendo...')
        self.__send_data(7)
        self.__set_update_today()





